@@ -42,13 +42,13 @@ to reduce overhead accessing the real controls in kernel drivers.
 
 */
 
+#include "control_local.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include "control_local.h"
 #ifdef HAVE_LIBPTHREAD
 #include <pthread.h>
 #endif
@@ -680,7 +680,7 @@ int snd_hctl_wait(snd_hctl_t *hctl, int timeout)
 	
 	npfds = snd_hctl_poll_descriptors_count(hctl);
 	if (npfds <= 0 || npfds >= 16) {
-		SNDERR("Invalid poll_fds %d\n", npfds);
+		SNDERR("Invalid poll_fds %d", npfds);
 		return -EIO;
 	}
 	pfd = alloca(sizeof(*pfd) * npfds);
@@ -689,14 +689,14 @@ int snd_hctl_wait(snd_hctl_t *hctl, int timeout)
 	if (err < 0)
 		return err;
 	if (err != npfds) {
-		SNDMSG("invalid poll descriptors %d\n", err);
+		SNDMSG("invalid poll descriptors %d", err);
 		return -EIO;
 	}
 	do {
 		pollio = 0;
 		err_poll = poll(pfd, npfds, timeout);
 		if (err_poll < 0) {
-			if (errno == EINTR && !CTLINABORT(hctl->ctl))
+			if (errno == EINTR && !CTLINABORT(hctl->ctl) && !(hctl->ctl->mode & SND_CTL_EINTR))
 				continue;
 			return -errno;
 		}
