@@ -224,6 +224,8 @@ struct snd_pcm_direct {
 	snd1_pcm_direct_nonblock
 #define snd_pcm_direct_async \
 	snd1_pcm_direct_async
+#define snd_pcm_direct_poll_descriptors \
+	snd1_pcm_direct_poll_descriptors
 #define snd_pcm_direct_poll_revents \
 	snd1_pcm_direct_poll_revents
 #define snd_pcm_direct_info \
@@ -264,6 +266,10 @@ struct snd_pcm_direct {
 	snd1_pcm_direct_set_chmap
 #define snd_pcm_direct_reset_slave_ptr \
 	snd1_pcm_direct_reset_slave_ptr
+#define snd_pcm_direct_check_xrun \
+	snd1_pcm_direct_check_xrun
+#define snd_pcm_direct_slave_recover \
+	snd1_pcm_direct_slave_recover
 
 int snd_pcm_direct_semaphore_create_or_connect(snd_pcm_direct_t *dmix);
 
@@ -345,10 +351,10 @@ snd_pcm_chmap_query_t **snd_pcm_direct_query_chmaps(snd_pcm_t *pcm);
 snd_pcm_chmap_t *snd_pcm_direct_get_chmap(snd_pcm_t *pcm);
 int snd_pcm_direct_set_chmap(snd_pcm_t *pcm, const snd_pcm_chmap_t *map);
 int snd_pcm_direct_slave_recover(snd_pcm_direct_t *direct);
-int snd_pcm_direct_client_chk_xrun(snd_pcm_direct_t *direct, snd_pcm_t *pcm);
+int snd_pcm_direct_check_xrun(snd_pcm_direct_t *direct, snd_pcm_t *pcm);
 int snd_timer_async(snd_timer_t *timer, int sig, pid_t pid);
 struct timespec snd_pcm_hw_fast_tstamp(snd_pcm_t *pcm);
-void snd_pcm_direct_reset_slave_ptr(snd_pcm_t *pcm, snd_pcm_direct_t *dmix);
+void snd_pcm_direct_reset_slave_ptr(snd_pcm_t *pcm, snd_pcm_direct_t *dmix, snd_pcm_uframes_t hw_ptr);
 
 struct snd_pcm_direct_open_conf {
 	key_t ipc_key;
@@ -365,3 +371,7 @@ struct snd_pcm_direct_open_conf {
 };
 
 int snd_pcm_direct_parse_open_conf(snd_config_t *root, snd_config_t *conf, int stream, struct snd_pcm_direct_open_conf *rec);
+
+int _snd_pcm_direct_new(snd_pcm_t **pcmp, snd_pcm_direct_t **_dmix, int type,
+			const char *name, struct snd_pcm_direct_open_conf *opts,
+			struct slave_params *params, snd_pcm_stream_t stream, int mode);
