@@ -26,11 +26,10 @@
  *
  */
 
-#include "bswap.h"
-#include <math.h>
 #include "pcm_local.h"
 #include "pcm_plugin.h"
-
+#include "bswap.h"
+#include <math.h>
 #include <sound/tlv.h>
 
 #ifndef PIC
@@ -114,7 +113,7 @@ static inline int MULTI_DIV_32x16(int a, unsigned short b)
 	y.i = 0;
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 	x.i = (unsigned short)v.s[0];
-	x.i *= b;
+	x.i *= (unsigned int)b;
 	y.s[0] = x.s[1];
 	y.i += (int)v.s[1] * b;
 #else
@@ -711,13 +710,13 @@ static int add_tlv_info(snd_pcm_softvol_t *svol, snd_ctl_elem_info_t *cinfo,
 			unsigned int *old_tlv, size_t old_tlv_size)
 {
 	unsigned int tlv[4];
-	if (sizeof(tlv) <= old_tlv_size && memcmp(tlv, old_tlv, sizeof(tlv)) == 0)
-		return 0;
 	tlv[SNDRV_CTL_TLVO_TYPE] = SND_CTL_TLVT_DB_SCALE;
 	tlv[SNDRV_CTL_TLVO_LEN] = 2 * sizeof(int);
 	tlv[SNDRV_CTL_TLVO_DB_SCALE_MIN] = (int)(svol->min_dB * 100);
 	tlv[SNDRV_CTL_TLVO_DB_SCALE_MUTE_AND_STEP] =
 		(int)((svol->max_dB - svol->min_dB) * 100 / svol->max_val);
+	if (sizeof(tlv) <= old_tlv_size && memcmp(tlv, old_tlv, sizeof(tlv)) == 0)
+		return 0;
 	return snd_ctl_elem_tlv_write(svol->ctl, &cinfo->id, tlv);
 }
 
@@ -983,8 +982,8 @@ int snd_pcm_softvol_open(snd_pcm_t **pcmp, const char *name,
 	return 0;
 }
 
-int _snd_pcm_parse_control_id(snd_config_t *conf, snd_ctl_elem_id_t *ctl_id,
-			      int *cardp, int *cchannels)
+static int _snd_pcm_parse_control_id(snd_config_t *conf, snd_ctl_elem_id_t *ctl_id,
+				     int *cardp, int *cchannels)
 {
 	snd_config_iterator_t i, next;
 	int iface = SND_CTL_ELEM_IFACE_MIXER;
@@ -1190,7 +1189,7 @@ int _snd_pcm_softvol_open(snd_pcm_t **pcmp, const char *name,
 			continue;
 		}
 		if (strcmp(id, "min_dB") == 0) {
-			err = snd_config_get_real(n, &min_dB);
+			err = snd_config_get_ireal(n, &min_dB);
 			if (err < 0) {
 				SNDERR("Invalid min_dB value");
 				return err;
@@ -1198,7 +1197,7 @@ int _snd_pcm_softvol_open(snd_pcm_t **pcmp, const char *name,
 			continue;
 		}
 		if (strcmp(id, "max_dB") == 0) {
-			err = snd_config_get_real(n, &max_dB);
+			err = snd_config_get_ireal(n, &max_dB);
 			if (err < 0) {
 				SNDERR("Invalid max_dB value");
 				return err;
