@@ -56,7 +56,7 @@ Events are read via snd_timer_read() function.
 The full featured examples with cross-links:
 
 \par Simple timer test program
-\ref example_test_timer "example code"
+\link example_test_timer example code \endlink
 \par
 This example shows opening a timer device and reading of timer events.
 
@@ -205,9 +205,15 @@ int snd_timer_open(snd_timer_t **timer, const char *name, int mode)
 	int err;
 
 	assert(timer && name);
-	err = snd_config_update_ref(&top);
-	if (err < 0)
-		return err;
+	if (_snd_is_ucm_device(name)) {
+		name = uc_mgr_alibcfg_by_device(&top, name);
+		if (name == NULL)
+			return -ENODEV;
+	} else {
+		err = snd_config_update_ref(&top);
+		if (err < 0)
+			return err;
+	}
 	err = snd_timer_open_noupdate(timer, top, name, mode);
 	snd_config_unref(top);
 	return err;
